package com.cultural.heritage.controller.book;


import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.clothesgrade.ClothesGradeAddRequest;
import com.cultural.heritage.model.dto.clothesgrade.ClothesGradeUpdateRequest;
import com.cultural.heritage.model.entity.ClothesGrade;
import com.cultural.heritage.model.entity.ClothesInfo;
import com.cultural.heritage.model.vo.clothesgrade.ClothesGradeVO;
import com.cultural.heritage.service.book.ClothesGradeService;
import com.cultural.heritage.service.book.ClothesInfoService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/clothesGrade")
@Slf4j
@Tag(name = "服装等级管理模块")
public class ClothesGradeController {


    @Resource
    private ClothesGradeService clothesGradeService;


    @Resource
    private ClothesInfoService clothesInfoService;


    /**
     * Web端管理员添加服装等级
     * @param clothesGradeAddRequest    服装等级添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员添加服装等级", description = "参数：服装等级添加请求体，权限：管理员(admin, boss)，方法名：addClothesGrade")
    public BaseResponse<Long> addClothesGrade(@RequestBody ClothesGradeAddRequest clothesGradeAddRequest) {
        if (clothesGradeAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        ClothesGrade clothesGrade = new ClothesGrade();
        BeanUtils.copyProperties(clothesGradeAddRequest, clothesGrade);
        // 校验
        clothesGradeService.validClothesGrade(clothesGrade, false);
        boolean save = clothesGradeService.save(clothesGrade);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(clothesGrade.getId());
    }



    /**
     * Web端管理员更新服装等级
     * @param clothesGradeUpdateRequest     服装等级更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员更新服装等级", description = "参数：服装等级更新请求体，权限：管理员(admin, boss)，方法名：updateClothesGrade")
    public BaseResponse<Boolean> updateClothesGrade(@RequestBody ClothesGradeUpdateRequest clothesGradeUpdateRequest) {
        if (clothesGradeUpdateRequest == null || clothesGradeUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        // 获取原有服装等级名称
        Long id = clothesGradeUpdateRequest.getId();
        ClothesGrade originClothesGrade = clothesGradeService.getById(id);
        String originClothesType = originClothesGrade.getClothesType();
        // 获取目标服装等级名称
        String targetClothesType = clothesGradeUpdateRequest.getClothesType();
        // 判空
        if (StringUtils.isAnyBlank(originClothesType, targetClothesType)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "服装等级参数为空");
        }
        // 更新原有服装等级下所有服装的等级
        UpdateWrapper<ClothesInfo> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("clothesType", originClothesType).set("clothesType", targetClothesType);
        boolean update = clothesInfoService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "服装等级名称更新失败");
        // 更新服装等级信息
        ClothesGrade clothesGrade = new ClothesGrade();
        BeanUtils.copyProperties(clothesGradeUpdateRequest, clothesGrade);
        // 校验
        clothesGradeService.validClothesGrade(clothesGrade, true);
        boolean result = clothesGradeService.updateById(clothesGrade);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服装等级信息更新失败");

        return ResultUtils.success(true);
    }


    /**
     * 删除服装等级
     * @param commonRequest     服装等级删除请求体
     * @return 是否删除成功
     */
    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员删除服装等级", description = "参数：服装等级删除请求体，权限：管理员(admin, boss)，方法名：deleteClothesGrade")
    public BaseResponse<Boolean> deleteClothesGrade(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        // 删除等级服装之前，先删除该等级服装下的所有商品
        clothesGradeService.deleteCurrentAllClothes(id);
        boolean result = clothesGradeService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


    /**
     * 查询所有服装等级
     * @return 服装等级列表
     */
    @PostMapping("/list")
    @Operation(summary = "Web端和小程序端查询服装等级", description = "参数：无，权限：所有人，方法名：listClothesGrade")
    public BaseResponse<List<ClothesGradeVO>> listClothesGrade() {
        List<ClothesGrade> clothesGradeList = clothesGradeService.list();
        List<ClothesGradeVO> clothesGradeVOS = clothesGradeList.stream().map(clothesGrade -> {
            ClothesGradeVO clothesGradeVO = new ClothesGradeVO();
            BeanUtils.copyProperties(clothesGrade, clothesGradeVO);
            return clothesGradeVO;
        }).toList();
        return ResultUtils.success(clothesGradeVOS);
    }

}
