package com.cultural.heritage.controller.good;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.cart.CartRecordAddRequest;
import com.cultural.heritage.model.dto.cart.CartRecordUpdateRequest;
import com.cultural.heritage.model.dto.order.BuySingleGoodVerifyRequest;
import com.cultural.heritage.model.entity.CartRecord;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.vo.cart.CartGoodVO;
import com.cultural.heritage.model.vo.cart.CartOrderVO;
import com.cultural.heritage.model.vo.cart.CartRecordVO;
import com.cultural.heritage.service.good.CartRecordService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.*;

@RestController
@RequestMapping("/cart")
@Slf4j
@Tag(name = "购物车信息模块")
public class CartRecordController {


    @Resource
    private CartRecordService cartRecordService;


    @Resource
    private GoodService goodService;


    @Resource
    private UserService userService;

    /**
     * 用户添加商品至购物车
     * @param cartRecordAddRequest  购物车记录添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "小程序端用户添加商品至购物车", description = "参数：购物车记录添加请求体，权限：所有人，方法名：addCartRecord")
    public BaseResponse<Boolean> addCartRecord(@RequestBody CartRecordAddRequest cartRecordAddRequest, HttpServletRequest request) {
        if (cartRecordAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 更新购物车的商品属性
        cartRecordService.updateCartGoodAttribute(cartRecordAddRequest, request);
        return ResultUtils.success(true);
    }


    /**
     * 小程序端用户更新购物车中的商品数量
     * @param cartRecordUpdateRequestList   购物车记录列表更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "小程序端用户更新购物车中的商品数量", description = "参数：购物车记录更新请求体，权限：所有人，方法名：updateCartRecord")
    public BaseResponse<Boolean> updateCartRecord(@RequestBody List<CartRecordUpdateRequest> cartRecordUpdateRequestList, HttpServletRequest request) {
        if (CollectionUtils.isEmpty(cartRecordUpdateRequestList)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        List<CartRecord> cartRecordList = cartRecordUpdateRequestList.stream().map(cartRecordUpdateRequest -> {
            Long id = cartRecordUpdateRequest.getId();
            CartRecord cart = cartRecordService.getById(id);
            ThrowUtils.throwIf(cart == null, ErrorCode.OPERATION_ERROR, "该项商品已被移除");
            CartRecord cartRecord = new CartRecord();
            BeanUtils.copyProperties(cartRecordUpdateRequest, cartRecord);
            Long goodId = cart.getGoodId();
            Good good = goodService.getById(goodId);
            ThrowUtils.throwIf(good == null || good.getIsShelves() == 0, ErrorCode.OPERATION_ERROR, "购物车商品更新失败");
            cartRecord.setSubtotal(good.getPrice().multiply(BigDecimal.valueOf(cartRecord.getQuantity())));
            cartRecord.setUserId(userId);
            cartRecord.setGoodId(goodId);
            // 校验
            cartRecordService.validCart(cartRecord, true);
            return cartRecord;
        }).toList();
        boolean result = cartRecordService.updateBatchById(cartRecordList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 用户删除购物车的物品
     * @param idList  删除的商品id列表
     * @return 是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "小程序端用户删除购物车的物品", description = "参数：购物车记录删除请求体(商品id列表，如[2,3,4])，权限：所有人，方法名：deleteCart")
    public BaseResponse<Boolean> deleteCart(@RequestBody List<Integer> idList, HttpServletRequest request) {
        if (CollectionUtils.isEmpty(idList)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        boolean result = cartRecordService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 小程序端用户查询购物车中的商品
     * @return 商品列表信息
     */
    @PostMapping("/list")
    @Operation(summary = "小程序端用户查询购物车中的商品", description = "参数：无，权限：所有人，方法名：listUserCartRecord")
    public BaseResponse<List<CartRecordVO>> listUserCartRecord(HttpServletRequest request) {
        // 根据用户id查询购物车记录
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        QueryWrapper<CartRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        queryWrapper.orderByDesc("id");
        List<CartRecord> list = cartRecordService.list(queryWrapper);
        // 封装用户购物车中的商品信息
        List<CartRecordVO> cartRecordVOList = cartRecordService.transformToCartRecordVOList(list);
        return ResultUtils.success(cartRecordVOList);
    }




    /**
     * (小程序端)从购物车中找出不合理的商品项
     * @return  存在问题的购物车id列表
     */
    @PostMapping("/submit/error")
    @Operation(summary = "(小程序端)从购物车中找出不合理的商品项", description = "参数:无，权限：所有人，方法名：listInvalidCartIds")
    public BaseResponse<List<Long>> listInvalidCartIds (HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        QueryWrapper<CartRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        List<CartRecord> cartRecords = cartRecordService.list(queryWrapper);
        // 找出存在问题的购物车id列表
        List<Long> invalidCartIds = cartRecordService.getInvalidCartIds(cartRecords);
        return ResultUtils.success(invalidCartIds);
    }





    /**
     * (小程序端)展示购物车提交订单页的商品信息
     * @param cartRecordIds    购物车id列表
     * @return  订单商品信息列表
     */
    @PostMapping("/submit/list")
    @Operation(summary = "(小程序端)展示购物车提交订单页的商品信息", description = "参数:商品id和购买数量信息，权限：所有人，方法名：listCartRecord")
    public BaseResponse<List<CartOrderVO>> listCartRecord(@RequestBody List<Long> cartRecordIds, HttpServletRequest request) {
        if (CollectionUtils.isEmpty(cartRecordIds)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        List<CartRecord> cartRecordList = cartRecordService.list();
        // 获取购物车列表信息
        List<CartRecord> cartRecords = cartRecordList.stream().filter(cartRecord -> {
            Long id = cartRecord.getId();
            return cartRecordIds.contains(id);
        }).toList();

        // 判断是否存在无效的购物车id
        if (cartRecords.size() < cartRecordIds.size()) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在无效的购物车id");
        }

        // 提取出购物车中的商品id列表
        List<Long> goodIds = cartRecords.stream().map(CartRecord::getGoodId).toList();
        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", goodIds);
        List<Good> goodList = goodService.list(queryWrapper);

        // 封装goodMap(根据商品id查询商品)
        Map<Long, Good> goodMap = new HashMap<>();
        for (Good good : goodList) {
            goodMap.put(good.getId(), good);
        }

        // 封装提交订单页的商品信息列表
        List<CartOrderVO> cartOrderVOList = new ArrayList<>();
        for (CartRecord cartRecord : cartRecords) {
            Integer quantity = cartRecord.getQuantity();
            Long goodId = cartRecord.getGoodId();
            Good good = goodMap.get(goodId);
            CartGoodVO cartGoodVO = new CartGoodVO();
            BeanUtils.copyProperties(good, cartGoodVO);
            CartOrderVO cartOrderVO = new CartOrderVO();
            cartOrderVO.setCartGoodVO(cartGoodVO);
            cartOrderVO.setQuantity(quantity);
            cartOrderVOList.add(cartOrderVO);
        }
        Collections.reverse(cartOrderVOList);
        return ResultUtils.success(cartOrderVOList);
    }





    /**
     * (小程序端)用户点击立即购买或者支付订单触发的校验
     * @param buySingleGoodVerifyRequest    商品id和购买数量
     * @return  是否通过校验
     */
    @PostMapping("/verify/good")
    @Operation(summary = "(小程序端)用户点击立即购买或者支付订单触发的校验", description = "参数:商品id和购买数量，权限：所有人，方法名：verifyPurchaseInfo")
    public BaseResponse<Boolean> verifyPurchaseInfo(@RequestBody BuySingleGoodVerifyRequest buySingleGoodVerifyRequest, HttpServletRequest request) {
        if (buySingleGoodVerifyRequest == null || buySingleGoodVerifyRequest.getGoodId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long goodId = buySingleGoodVerifyRequest.getGoodId();
        Integer quantity = buySingleGoodVerifyRequest.getQuantity();
        Good good = goodService.getById(goodId);
        ThrowUtils.throwIf(good == null || good.getIsShelves() == 0, ErrorCode.SYSTEM_ERROR, "商品已下架或不存在");
        ThrowUtils.throwIf(quantity > good.getInventory(), ErrorCode.SYSTEM_ERROR, "商品库存不足");
        return ResultUtils.success(true);
    }

}
