package com.cultural.heritage.controller.good;


import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonBatchRequest;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.coupon.CouponAddRequest;
import com.cultural.heritage.model.dto.coupon.CouponQueryRequest;
import com.cultural.heritage.model.dto.coupon.CouponUpdateRequest;
import com.cultural.heritage.model.entity.Coupon;
import com.cultural.heritage.model.entity.Exchange;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.entity.UserCoupon;
import com.cultural.heritage.model.vo.coupon.CouponVO;
import com.cultural.heritage.model.vo.coupon.UserCouponVO;
import com.cultural.heritage.service.good.CouponService;
import com.cultural.heritage.service.good.ExchangeService;
import com.cultural.heritage.service.good.UserCouponService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@RestController
@RequestMapping("/coupon")
@Slf4j
@Tag(name = "优惠券管理模块")
public class CouponController {

    @Resource
    private CouponService couponService;


    @Resource
    private UserService userService;


    @Resource
    private ExchangeService exchangeService;


    @Resource
    private UserCouponService userCouponService;


    /**
     * 添加优惠券
     * @param couponAddRequest  优惠券添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加优惠券", description = "参数：优惠券添加请求体，权限：管理员(admin, boss)，方法名：addCoupon")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addCoupon(@RequestBody CouponAddRequest couponAddRequest) {
        if (couponAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Coupon coupon = new Coupon();
        BeanUtils.copyProperties(couponAddRequest, coupon);
        // 填充优惠券有效开始日期
        String startTime = DateUtil.format(DateUtil.date(), "yyyy-MM-dd 00:00:00");
        coupon.setStartTime(startTime);

        // 比较优惠券有效截止日期
        String endTime = coupon.getEndTime();
        String currentTime = DateUtil.now();
        if (endTime.compareTo(currentTime) <= 0) {
            coupon.setStatus("已过期");
        } else {
            coupon.setStatus("可用");
        }
        // 校验
        couponService.validCoupon(coupon, false);
        boolean result = couponService.save(coupon);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "优惠券添加失败");
        return ResultUtils.success(true);
    }



    /**
     * 更新优惠券
     * @param couponUpdateRequest   优惠券更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新优惠券", description = "参数：优惠券更新请求体，权限：管理员(admin, boss)，方法名：updateCoupon")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateCoupon(@RequestBody CouponUpdateRequest couponUpdateRequest) {
        if (couponUpdateRequest == null || couponUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = couponUpdateRequest.getId();
        Coupon coupon = couponService.getById(id);
        ThrowUtils.throwIf(coupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");
        BeanUtils.copyProperties(couponUpdateRequest, coupon);

        // 比较优惠券有效截止日期
        String endTime = coupon.getEndTime();
        String currentTime = DateUtil.now();
        if (endTime.compareTo(currentTime) <= 0) {
            coupon.setStatus("已过期");
        } else {
            coupon.setStatus("可用");
        }

        // 校验
        couponService.validCoupon(coupon, true);
        boolean result = couponService.saveOrUpdate(coupon);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "优惠券更新失败");

        QueryWrapper<UserCoupon> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("couponId", coupon.getId());
        List<UserCoupon> userCouponList = userCouponService.list(queryWrapper);
        for (UserCoupon userCoupon : userCouponList) {
            // 更新用户优惠券中的优惠券信息
            CouponVO couponVO = new CouponVO();
            BeanUtils.copyProperties(coupon, couponVO);
            userCoupon.setCouponVO(couponVO);
            userCoupon.setStatus(coupon.getStatus());
        }
        boolean update = userCouponService.updateBatchById(userCouponList);
        ThrowUtils.throwIf(!CollectionUtils.isEmpty(userCouponList) && !update, ErrorCode.OPERATION_ERROR, "优惠券更新失败");
        return ResultUtils.success(true);
    }



    /**
     * 删除优惠券
     * @param couponDeleteRequest   优惠券删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "Web端管理员删除优惠券", description = "参数：优惠券删除请求体，权限：管理员(admin, boss)，方法名：deleteCoupon")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteCoupon(@RequestBody CommonRequest couponDeleteRequest) {
        if (couponDeleteRequest == null || couponDeleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = couponDeleteRequest.getId();
        boolean result = couponService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "优惠券删除失败");
        return ResultUtils.success(true);
    }



    /**
     * 批量删除优惠券
     * @param commonBatchRequest   优惠券删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/del/Batch")
    @Operation(summary = "Web端管理员批量删除优惠券", description = "参数：优惠券删除请求体，权限：管理员(admin, boss)，方法名：deleteCoupon")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteCoupon(@RequestBody CommonBatchRequest commonBatchRequest) {
        if (commonBatchRequest == null || CollectionUtils.isEmpty(commonBatchRequest.getIdList())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数为null或数组为空");
        }
        List<Long> idList = commonBatchRequest.getIdList();
        boolean result = couponService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "优惠券批量删除失败");
        return ResultUtils.success(true);
    }



    /**
     * Web端管理员分页查询优惠券
     * @param couponQueryRequest 优惠券查询请求体
     * @return 优惠券列表
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询优惠券", description = "参数：优惠券查询请求体, 排序字段 sortField: conditionAmount requirePoints totalNum residueNum limitNum id，权限：管理员(admin, boss)，方法名：listCouponVOByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<Coupon>> listCouponVOByPage(@RequestBody CouponQueryRequest couponQueryRequest) {
        if (couponQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        // 对优惠券是否过期预处理
        long current = couponQueryRequest.getCurrent();
        long pageSize = couponQueryRequest.getPageSize();
        Page<Coupon> page = couponService.page(new Page<>(current, pageSize),
                couponService.getQueryWrapper(couponQueryRequest));
        List<Coupon> couponList = page.getRecords();
        for (Coupon coupon : couponList) {
            // 比较优惠券有效截止日期
            String endTime = coupon.getEndTime();
            String currentTime = DateUtil.now();
            if (endTime.compareTo(currentTime) <= 0) {
                coupon.setStatus("已过期");
            } else {
                coupon.setStatus("可用");
            }
        }
        boolean update = couponService.updateBatchById(couponList);
        ThrowUtils.throwIf(!CollectionUtils.isEmpty(couponList) && !update, ErrorCode.OPERATION_ERROR, "优惠券状态更新失败");
        page.setRecords(couponList);
        return ResultUtils.success(page);
    }



    /**
     * 用户删除兑换记录
     * @param commonRequest  兑换记录删除请求体
     * @return  是否兑换成功
     */
    @PostMapping("/delRecord")
    @Operation(summary = "小程序端用户删除兑换记录", description = "参数：兑换记录删除请求体，权限：管理员(admin, boss)，方法名：deleteExchangeRecord")
    public BaseResponse<Boolean> deleteExchangeRecord(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long id = commonRequest.getId();
        boolean result = exchangeService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "兑换记录删除失败");
        return ResultUtils.success(true, "删除兑换记录成功");
    }



    /**
     * 小程序端用户积分兑换优惠券
     * @param commonRequest 兑换记录添加请求体
     * @return
     */
    @PostMapping("/exchange")
    @Operation(summary = "小程序端用户积分兑换优惠券", description = "参数：兑换记录添加请求体，权限：所有人，方法名：pointsExchangeCoupon")
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> pointsExchangeCoupon(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        // 获取当前用户信息
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Integer points = loginUser.getPoints();

        // 获取当前优惠券信息
        Long id = commonRequest.getId();
        Coupon coupon = couponService.getById(id);
        ThrowUtils.throwIf(coupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");

        // 判断用户积分是否充足
        Integer requirePoints = coupon.getRequirePoints();
        if (points < requirePoints) throw new BusinessException(ErrorCode.OPERATION_ERROR, "积分不足");

        // 插入一条兑换记录
        CouponVO couponVO = new CouponVO();
        BeanUtils.copyProperties(coupon, couponVO);
        Exchange exchange = new Exchange();
        exchange.setUserId(userId);
        exchange.setCouponVO(couponVO);
        boolean save = exchangeService.save(exchange);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "兑换记录插入失败");

        // 更新用户优惠券表
        QueryWrapper<UserCoupon> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId).eq("couponId", coupon.getId());
        UserCoupon userCoupon = userCouponService.getOne(queryWrapper);
        if (userCoupon == null) {
            UserCoupon userCouponSave = new UserCoupon();
            userCouponSave.setUserId(userId);
            userCouponSave.setCouponId(coupon.getId());
            userCouponSave.setCouponVO(couponVO);
            userCouponSave.setQuantity(1);
            userCouponSave.setStatus("可用");
            boolean saveSuccess = userCouponService.save(userCouponSave);
            ThrowUtils.throwIf(!saveSuccess, ErrorCode.OPERATION_ERROR, "用户优惠券插入失败");
        } else {
            UpdateWrapper<UserCoupon> updateWrapper = new UpdateWrapper<>();
            updateWrapper.eq("id", userCoupon.getId()).set("quantity", userCoupon.getQuantity() + 1);
            boolean update = userCouponService.update(updateWrapper);
            ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "用户优惠券更新失败");
        }

        // 更新用户积分
        loginUser.setPoints(loginUser.getPoints() - requirePoints);
        boolean isSuccess = userService.updateById(loginUser);
        ThrowUtils.throwIf(!isSuccess, ErrorCode.OPERATION_ERROR, "更新用户积分信息失败");

        return ResultUtils.success(true, "兑换成功");
    }





    /**
     * 小程序端用户查看优惠券
     * @return 当前用户拥有的优惠券列表
     */
    @GetMapping("/list/my")
    @Operation(summary = "小程序端用户查看优惠券", description = "参数：无，权限：所有人，方法名：pointsExchangeCoupon")
    public BaseResponse<List<UserCouponVO>> listMyCoupon(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        QueryWrapper<Exchange> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        List<UserCoupon> userCouponList = userCouponService.list();

        for (UserCoupon userCoupon : userCouponList) {
            CouponVO couponVO = userCoupon.getCouponVO();
            // 比较优惠券有效截止日期
            String endTime = couponVO.getEndTime();
            String currentTime = DateUtil.now();
            if (endTime.compareTo(currentTime) <= 0) {
                couponVO.setStatus("已过期");
                userCoupon.setStatus("已过期");
            } else {
                couponVO.setStatus("可用");
                userCoupon.setStatus("可用");
            }
        }

        boolean update = userCouponService.updateBatchById(userCouponList);
        ThrowUtils.throwIf(!CollectionUtils.isEmpty(userCouponList) && !update, ErrorCode.OPERATION_ERROR, "优惠券状态更新失败");

        List<UserCouponVO> userCouponVOS = userCouponList.stream().map(userCoupon -> {
            UserCouponVO userCouponVO = new UserCouponVO();
            BeanUtils.copyProperties(userCoupon, userCouponVO);
            return userCouponVO;
        }).toList();
        return ResultUtils.success(userCouponVOS);
    }





    /**
     * Web端管理员上(下)架优惠券
     * @param commonRequest 优惠券id
     * @return 是否更新成功
     */
    @PostMapping("/shelves")
    @Operation(summary = "Web端管理员上(下)架优惠券", description = "参数：优惠券上架状态更新请求体，权限：管理员(admin, boss)，方法名：deleteExchangeRecord")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateCouponShelvesStatus(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        Coupon coupon = couponService.getById(id);
        ThrowUtils.throwIf(coupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");

        Integer status = coupon.getIsShelves() == 0 ? 1 : 0;
        UpdateWrapper<Coupon> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("isShelves", status);
        boolean update = couponService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "上架状态更新失败");
        return ResultUtils.success(true);
    }





//    /**
//     * 小程序端用户查看兑换记录
//     * @param commonRequest 优惠券id
//     * @return 是否更新成功
//     */
//    @PostMapping("/get/record")
//    @Operation(summary = "Web端管理员上(下)架优惠券", description = "参数：优惠券上架状态更新请求体，权限：管理员(admin, boss)，方法名：deleteExchangeRecord")
//    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
//    public BaseResponse<Boolean> updateCouponShelvesStatus(@RequestBody CommonRequest commonRequest) {
//        if (commonRequest == null || commonRequest.getId() <= 0) {
//            throw new BusinessException(ErrorCode.PARAMS_ERROR);
//        }
//        Long id = commonRequest.getId();
//        Coupon coupon = couponService.getById(id);
//        ThrowUtils.throwIf(coupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");
//
//        Integer status = coupon.getIsShelves() == 0 ? 1 : 0;
//        UpdateWrapper<Coupon> updateWrapper = new UpdateWrapper<>();
//        updateWrapper.eq("id", id);
//        updateWrapper.set("isShelves", status);
//        boolean update = couponService.update(updateWrapper);
//        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "上架状态更新失败");
//        return ResultUtils.success(true);
//    }


}
