package com.cultural.heritage.controller.good;


import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 节日管理接口
 */
@RestController
@RequestMapping("/festival")
@Slf4j
@Tag(name = "节日管理模块")
public class FestivalController {


    @Resource
    private RedisTemplate<String, Object> redisTemplate;

    // 节日集合的键
    private static final String FESTIVAL_KEY = "festivalList";


    /**
     * Web端管理员添加节日项
     * @param value 添加的节日项
     */
    @GetMapping("/add")
    @Operation(summary = "Web端管理员添加节日项", description = "参数：节日项名称，权限：管理员(admin, boss)，方法名：addFestivalElement")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addFestivalElement(@RequestParam String value) {
        if (StringUtils.isBlank(value)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        if (isElementExistInList(value)) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "该节日项已存在");
        }
        redisTemplate.opsForList().rightPush(FESTIVAL_KEY, value);
        return ResultUtils.success(true);
    }


    /**
     * Web端管理员查询节日集合
     * @return 节日列表
     */
    @GetMapping("/get")
    @Operation(summary = "Web端管理员查询节日集合", description = "参数：无，权限：管理员(admin, boss)，方法名：getFestivalList")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<List<Object>> getFestivalList() {
        List<Object> festivalList = redisTemplate.opsForList().range(FESTIVAL_KEY, 0, -1);
        return ResultUtils.success(festivalList);
    }


    /**
     * Web端管理员删除节日项
     * @param value 删除的节日项
     * @return
     */
    @GetMapping("/delete")
    @Operation(summary = "Web端管理员删除节日项", description = "参数：无，权限：管理员(admin, boss)，方法名：deleteFestivalElement")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteFestivalElement(@RequestParam String value) {
        if (!isElementExistInList(value)) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "该节日项不存在");
        }
        redisTemplate.opsForList().remove(FESTIVAL_KEY, 1, value);
        return ResultUtils.success(true);
    }



    // 检查元素是否存在
    private boolean isElementExistInList(@RequestParam String value) {
        List<Object> festivalList = redisTemplate.opsForList().range(FESTIVAL_KEY, 0, -1);
        return festivalList != null && festivalList.contains(value);
    }


}
